@@ -11,6 +11,7 @@ use File::Temp;
 use File::Basename;
 use File::Copy::Recursive;
 use Module::CoreList;
+use Module::CPANfile;
 
 sub new {
     my $class = shift;
@@ -57,16 +58,17 @@ sub cmd_install {
 
     if (@args) {
         $self->install(@args);
-    } else {
-        $self->install("--installdeps", ".");
+        return;
     }
 
+    my $cpanfile = File::Temp->new->filename;
+    $self->requirements_to_cpanfile->save($cpanfile);
+    $self->install("--installdeps", "--cpanfile", $cpanfile, ".");
+
     my @artifacts;
     $self->resolve(sub {
         my($artifact, $depth) = @_;
-        if ($depth == 0) {
-            printf "Using %s (%s)\n", $artifact->package, $artifact->version || '0';
-        }
+        printf "Using %s (%s)\n", $artifact->package, $artifact->version || '0';
         push @artifacts, $artifact;
     });
 
@@ -76,9 +78,7 @@ sub cmd_install {
 sub is_core {
     my($self, $module, $want_version) = @_;
 
-    unless (exists $Module::CoreList::version{$]+0}{$module}) {
-        return;
-    }
+    return unless exists $Module::CoreList::version{$]+0}{$module};
 
     CPAN::Meta::Requirements->from_string_hash({ $module => $want_version })
         ->accepts_module($module, $Module::CoreList::version{$]+0}{$module} || '0');
@@ -89,7 +89,8 @@ sub install {
 
     my $dir = File::Temp::tempdir(CLEANUP => 1);
     local $ENV{PERL_CPANM_HOME} = $dir;
-    system $^X, "-S", "cpanm", "--notest", "-L", $self->base_dir, @args if @args;
+    local $ENV{PERL_CPANM_OPT};
+    system $^X, "-S", "cpanm", "--quiet", "--notest", "-L", $self->base_dir, @args if @args;
 
     for my $ent (glob "$dir/latest-build/*") {
         next unless -d $ent;
@@ -133,7 +134,7 @@ sub cmd_list {
     $self->resolve(sub { push @artifacts, $_[0] });
 
     for my $artifact (sort { $a->package cmp $b->package } @artifacts) {
-        printf "%s (%s) in %s\n", $artifact->package, $artifact->version || '0', $artifact->path;
+        printf "%s (%s) in %s\n", $artifact->package, $artifact->version || '0', $artifact->distname;
     }
 }
 
@@ -142,7 +143,7 @@ sub cmd_tree {
 
     $self->resolve(sub {
         my($artifact, $depth) = @_;
-        printf "%s%s (%s) in %s\n", (" " x $depth), $artifact->package, $artifact->version || '0', $artifact->path;
+        printf "%s%s (%s) in %s\n", (" " x $depth), $artifact->package, $artifact->version || '0', $artifact->distname;
     });
 
 }
@@ -179,7 +180,7 @@ Last-Updated: @{[ scalar localtime ]}
 
 EOF
 
-    for my $p (@packages) {
+    for my $p (sort { $a->{package} cmp $b->{package} } @packages) {
         print $fh sprintf "%s %s  %s\n", pad($p->{package}, 32), pad($p->{version}, 10, 1), $p->{pathname};
     }
 }
@@ -197,9 +198,6 @@ sub pad {
 sub try_snapshot {
     my $self = shift;
 
-    # TODO loading snapshot without install causes artifacts not found
-    return;
-
     if (my $cpanfile = $self->try_cpanfile) {
         return "$cpanfile.snapshot" if -e "$cpanfile.snapshot";
     }
@@ -213,51 +211,63 @@ sub try_cpanfile {
     return;
 }
 
-sub build_requirements {
-    my($self, @args) = @_;
+sub requirements {
+    my $self = shift;
+    $self->{requirements} ||= $self->build_requirements;
+}
 
-    # TODO support more explicit way to load the list of modules
+sub build_requirements {
+    my $self = shift;
+
+    my $cpanfile = $self->try_cpanfile
+      or Carp::croak "Could not locate 'cpanfile' to load module list.";
+
+    my $requirements = Module::CPANfile->load($cpanfile)
+      ->prereqs->merged_requirements(['runtime', 'test'],['requires']);
 
     if (my $snapshot = $self->try_snapshot) {
         require Carton::Snapshot;
         my $snapshot = Carton::Snapshot->new(path => $snapshot);
-        return $self->snapshot_to_requirements($snapshot);
-    }
-
-    if (my $cpanfile = $self->try_cpanfile) {
-        require Module::CPANfile;
-        return Module::CPANfile->load($cpanfile)->prereqs->merged_requirements(['runtime'],['requires']);
-    }
-
-    return;
-}
-
-sub snapshot_to_requirements {
-    my($self, $snapshot) = @_;
-
-    $snapshot->load;
-
-    my $requirements = CPAN::Meta::Requirements->new;
-    for my $package ($snapshot->packages) {
-        if ($package->version && $package->version ne 'undef') {
-            $requirements->exact_version($package->name, $package->version);
-        } else {
-            $requirements->add_minimum($package->name, '0');
-        }
+        $self->apply_snapshot($requirements, $snapshot);
     }
 
     $requirements;
 }
 
+sub requirements_to_cpanfile {
+    my $self = shift;
+    Module::CPANfile->from_prereqs({ runtime => { requires => $self->requirements->as_string_hash } });
+}
+
+sub apply_snapshot {
+    my($self, $requirements, $snapshot) = @_;
+    $snapshot->load;
+    $self->apply_snapshot_recursively($requirements, $snapshot, [$requirements->required_modules]);
+}
+
+sub apply_snapshot_recursively {
+    my($self, $requirements, $snapshot, $modules) = @_;
+
+    for my $module (@$modules) {
+        my $dist = $snapshot->find($module) or next;
+        my $version = $dist->version_for($module);
+        # FIXME in carmel update, conflicting version requirement should be ignored
+        $requirements->exact_version($module, $version);
+        $requirements->add_requirements($dist->requirements);
+        $self->apply_snapshot_recursively($requirements, $snapshot, [$dist->requirements->required_modules]);
+    }
+}
+
 sub resolve_recursive {
     my($self, $root_reqs, $requirements, $seen, $cb, $depth) = @_;
 
     for my $module (sort $requirements->required_modules) {
         next if $module eq 'perl';
 
-        my $want_version = $requirements->requirements_for_module($module);
+        my $want_version = $root_reqs->requirements_for_module($module);
         next if $self->is_core($module, $want_version);
 
+        # FIXME there's a chance different version of the same module can be loaded here
         my $artifact = $self->repo->find($module, $want_version)
           or die "Could not find an artifact for $module => $want_version\nYou need to run `carmel install` first to get the modules installed and artifacts built.\n";
 
@@ -273,10 +283,7 @@ sub resolve_recursive {
 
 sub resolve {
     my($self, $cb) = @_;
-
-    my $requirements = $self->build_requirements
-      or Carp::croak "Could not locate 'cpanfile' to load module list.";
-    $self->resolve_recursive($requirements, $requirements, {}, $cb, 0);
+    $self->resolve_recursive($self->requirements, $self->requirements->clone, {}, $cb, 0);
 }
 
 sub env {
